import { EditorView } from '@codemirror/view';
import { LanguageSupport } from '@codemirror/language';
import { Extension } from '@codemirror/state';
import { Properties } from 'csstype';
import { MapStore } from 'nanostores';

/**
Describes a problem or hint for a piece of code.
*/
interface Diagnostic {
    /**
    The start position of the relevant text.
    */
    from: number;
    /**
    The end position. May be equal to `from`, though actually
    covering text is preferable.
    */
    to: number;
    /**
    The severity of the problem. This will influence how it is
    displayed.
    */
    severity: "info" | "warning" | "error";
    /**
    An optional source string indicating where the diagnostic is
    coming from. You can put the name of your linter here, if
    applicable.
    */
    source?: string;
    /**
    The message associated with this diagnostic.
    */
    message: string;
    /**
    An optional custom rendering function that displays the message
    as a DOM node.
    */
    renderMessage?: () => Node;
    /**
    An optional array of actions that can be taken on this
    diagnostic.
    */
    actions?: readonly Action$1[];
}
/**
An action associated with a diagnostic.
*/
interface Action$1 {
    /**
    The label to show to the user. Should be relatively short.
    */
    name: string;
    /**
    The function to call when the user activates this action. Is
    given the diagnostic's _current_ position, which may have
    changed since the creation of the diagnostic, due to editing.
    */
    apply: (view: EditorView, from: number, to: number) => void;
}

/**
 * Actions can return an object containing the two properties defined in this interface. Both are optional.
 * - update: An action can have a parameter. This method will be called whenever that parameter changes,
 *   immediately after Svelte has applied updates to the markup.
 * - destroy: Method that is called after the element is unmounted
 *
 * Additionally, you can specify which additional attributes and events the action enables on the applied element.
 * This applies to TypeScript typings only and has no effect at runtime.
 *
 * Example usage:
 * ```ts
 * interface Attributes {
 * 	newprop?: string;
 *	'on:event': (e: CustomEvent<boolean>) => void;
 * }
 *
 * export function myAction(node: HTMLElement, parameter: Parameter): ActionReturn<Parameter, Attributes> {
 *   // ...
 *   return {
 *     update: (updatedParameter) => {...},
 *     destroy: () => {...}
 *   };
 * }
 * ```
 *
 * Docs: https://svelte.dev/docs#template-syntax-element-directives-use-action
 */
interface ActionReturn<Parameter = any, Attributes extends Record<string, any> = Record<never, any>> {
    update?: (parameter: Parameter) => void;
    destroy?: () => void;
    /**
     * ### DO NOT USE THIS
     * This exists solely for type-checking and has no effect at runtime.
     * Set this through the `Attributes` generic instead.
     */
    $$_attributes?: Attributes;
}
/**
 * Actions are functions that are called when an element is created.
 * You can use this interface to type such actions.
 * The following example defines an action that only works on `<div>` elements
 * and optionally accepts a parameter which it has a default value for:
 * ```ts
 * export const myAction: Action<HTMLDivElement, { someProperty: boolean }> = (node, param = { someProperty: true }) => {
 *   // ...
 * }
 * ```
 * You can return an object with methods `update` and `destroy` from the function and type which additional attributes and events it has.
 * See interface `ActionReturn` for more details.
 *
 * Docs: https://svelte.dev/docs#template-syntax-element-directives-use-action
 */
interface Action<Element = HTMLElement, Parameter = any, Attributes extends Record<string, any> = Record<never, any>> {
    <Node extends Element>(node: Node, parameter?: Parameter): void | ActionReturn<Parameter, Attributes>;
}

type MaybePromise<T> = T | Promise<T>;
type Styles = {
    [val: string]: Properties | Styles;
};
type Options = {
    value: string;
    setup?: 'basic' | 'minimal';
    lang?: LanguageSupport | string;
    langMap?: Record<string, () => MaybePromise<LanguageSupport>>;
    useTabs?: boolean;
    styles?: Styles;
    tabSize?: number;
    theme?: Extension;
    diagnostics?: Diagnostic[];
    extensions?: Extension[];
    instanceStore?: MapStore<CodemirrorInstance>;
};
type CodemirrorInstance = {
    view: EditorView | null;
    extensions: Extension | null;
    value: string | null;
};
declare const withCodemirrorInstance: () => MapStore<CodemirrorInstance>;
declare const codemirror: Action<HTMLElement, Options, {
    'on:codemirror:change': (e: CustomEvent<string>) => void;
}>;

export { codemirror, withCodemirrorInstance };
